@@ -436,14 +436,13 @@ differ.splitIntoWords_ = function(line) {
     return SYM;
   };
 
-  // TODO: consider putting each whitespace char into its own word.
   // Single words can be [A-Z][a-z]+, [A-Z]+, [a-z]+, [0-9]+ or \s+.
   var words = [];
   var lastType = -1;
   for (var i = 0; i < line.length; i++) {
     var c = line.charAt(i);
     var ct = charType(c);
-    if (ct == lastType && ct != SYM ||
+    if (ct == lastType && ct != SYM && ct != WS ||
         ct == LC && lastType == UC && words[words.length - 1].length == 1) {
       words[words.length - 1] += c;
     } else {
@@ -605,6 +604,69 @@ differ.buildView = function(beforeText, afterText, userParams) {
   return d.buildView_();
 };
 
+/**
+ * Returns a valid HighlightJS language based on a file name/path.
+ * If it can't guess a language, returns null.
+ */
+differ.guessLanguageUsingFileName = function(name) {
+  var lang = (function() {
+    var m = /\.([^.]+)$/.exec(name);
+    if (m) {
+      var ext = m[1];
+      if (ext == 'py') return 'python';
+      if (ext == 'sh') return 'bash';
+      if (ext == 'md') return 'markdown';
+      return m[1].toLowerCase();
+    };
+
+    // Highlighting based purely on file name, e.g. "Makefile".
+    m = /(?:.*\/)?([^\/]*)$/.exec(name);
+    if (m && m[1] == 'Makefile') {
+      return 'makefile';
+    }
+    return null;
+  })();
+  if (!lang || !hljs.getLanguage(lang)) {
+    return null;
+  } else {
+    return lang;
+  }
+};
+
+/**
+ * Guess a language based on a file's contents.
+ * This always returns a valid HighlightJS language. It considers the shebang
+ * line (if present) and then falls back to HighlightJS's keyword-based
+ * guessing.
+ */
+differ.guessLanguageUsingContents = function(contents) {
+  // First check for a shebang line.
+  var firstLine = contents.substring(0, contents.indexOf('\n'));
+  if (firstLine.substring(0, 2) == '#!') {
+    var processor = firstLine.substring(2);
+    if (processor == '/bin/bash') return 'bash';
+    if (processor == '/bin/sh') return 'bash';
+
+    var options = {
+        'python': 'python',
+        'perl': 'perl',
+        'ruby': 'ruby',
+        'node': 'javascript'
+    };
+    for (var interpreter in options) {
+      var lang = options[interpreter];
+      if (processor.indexOf(interpreter) >= 0) {
+        return lang;
+      }
+    }
+  }
+
+  // Now let HighlightJS guess.
+  var guess = hljs.highlightAuto(contents);
+  var lang = guess.language;
+  return lang;
+};
+
 return differ;
 
 })();
