@@ -1,30 +1,41 @@
 /** @jsx React.DOM */
+var Route = ReactRouter.Route;
+var Routes = ReactRouter.Routes;
+var Link = ReactRouter.Link;
 
 IMAGE_DIFF_MODES = ['side-by-side', 'blink', 'onion-skin', 'swipe'];
 
 // Webdiff application root.
 var Root = React.createClass({
   propTypes: {
-    filePairs: React.PropTypes.array.isRequired
+    filePairs: React.PropTypes.array.isRequired,
+    initiallySelectedIndex: React.PropTypes.number,
+    params: React.PropTypes.object
   },
+  mixins: [ReactRouter.Navigation],
   getInitialState: () => ({
-    selectedFileIndex: 0,
     imageDiffMode: 'side-by-side'
   }),
-  fileChangeHandler: function(idx) {
-    this.setState({selectedFileIndex: idx});
+  selectIndex: function(idx) {
+    this.transitionTo('pair', {index:idx});
+  },
+  getIndex: function() {
+    var idx = this.props.params.index;
+    if (idx == null) idx = this.props.initiallySelectedIndex;
+    return Number(idx);
   },
   changeImageDiffModeHandler: function(mode) {
     this.setState({imageDiffMode: mode});
   },
   render: function() {
-    var filePair = this.props.filePairs[this.state.selectedFileIndex];
+    var idx = this.getIndex(),
+        filePair = this.props.filePairs[idx];
 
     return (
       <div>
-        <FileSelector selectedFileIndex={this.state.selectedFileIndex}
+        <FileSelector selectedFileIndex={idx}
                       filePairs={this.props.filePairs}
-                      fileChangeHandler={this.fileChangeHandler} />
+                      fileChangeHandler={this.selectIndex} />
         <DiffView filePair={filePair}
                   imageDiffMode={this.state.imageDiffMode}
                   changeImageDiffModeHandler={this.changeImageDiffModeHandler} />
@@ -34,14 +45,14 @@ var Root = React.createClass({
   componentDidMount: function() {
     $(document).on('keydown', (e) => {
       if (!isLegitKeypress(e)) return;
-      var idx = this.state.selectedFileIndex;
+      var idx = this.getIndex();
       if (e.keyCode == 75) {  // j
         if (idx > 0) {
-          this.setState({selectedFileIndex: idx - 1});
+          this.selectIndex(idx - 1);
         }
       } else if (e.keyCode == 74) {  // k
         if (idx < this.props.filePairs.length - 1) {
-          this.setState({selectedFileIndex: idx + 1});
+          this.selectIndex(idx + 1);
         }
       } else if (e.keyCode == 83) {  // s
         this.setState({imageDiffMode: 'side-by-side'});
@@ -192,6 +203,10 @@ var ImageDiffModeSelector = React.createClass({
     changeHandler: React.PropTypes.func.isRequired
   },
   render: function() {
+    if (isOneSided(this.props.filePair)) {
+      return null;  // Only "side-by-side" makes sense for one-sided diffs.
+    }
+
     // Returns the text, optionally wrapped in a link and/or <b> tag.
     var linkOrB = (isLink, isB, val, text) => {
       var inner = isB ? <b>{text}</b> : text;
@@ -237,13 +252,32 @@ var DiffView = React.createClass({
   }
 });
 
+// A "no changes" sign which only appears when applicable.
+var NoChanges = React.createClass({
+  propTypes: {
+    filePair: React.PropTypes.object.isRequired
+  },
+  render: function() {
+    if (this.props.filePair.no_changes) {
+      return <div className="no-changes">(No Changes)</div>;
+    } else {
+      return null;
+    }
+  }
+});
+
 // A side-by-side diff of source code.
 var CodeDiff = React.createClass({
   propTypes: {
     filePair: React.PropTypes.object.isRequired
   },
   render: function() {
-    return <div key={this.props.filePair.idx}>Loading&hellip;</div>;
+    return (
+      <div>
+        <NoChanges filePair={this.props.filePair} />
+        <div ref="codediff" key={this.props.filePair.idx}>Loading&hellip;</div>
+      </div>
+    );
   },
   renderDiff: function() {
     // Either side can be empty (i.e. an add or a delete), in which case
@@ -259,7 +293,7 @@ var CodeDiff = React.createClass({
     $.when(beforeDeferred, afterDeferred).done((before, after) => {
       if (!this.isMounted()) return;
       // Call out to codediff.js to construct the side-by-side diff.
-      $(this.getDOMNode()).empty().append(
+      $(this.refs.codediff.getDOMNode()).empty().append(
           renderDiff(pair.a, pair.b, before[0], after[0]));
     })
     .fail((e) => alert("Unable to get diff!"));
@@ -295,12 +329,15 @@ var ImageDiff = React.createClass({
   },
   render: function() {
     var mode = this.props.imageDiffMode;
+    if (isOneSided(this.props.filePair)) {
+      mode = 'side-by-side';  // Only one that makes sense for one-sided diffs.
+    }
     var component = {
       'side-by-side': ImageSideBySide,
       'blink': ImageBlinker,
       'onion-skin': ImageOnionSkin,
       'swipe': ImageSwipe
-    }[this.props.imageDiffMode];
+    }[mode];
     var image = component({
       filePair: this.props.filePair,
       shrinkToFit: this.state.shrinkToFit
@@ -315,6 +352,7 @@ var ImageDiff = React.createClass({
         <label htmlFor="shrink-to-fit"> Shrink to fit</label>
       </div>
       <div className={'image-diff ' + mode}>
+        <NoChanges filePair={this.props.filePair} />
         {image}
       </div>
     </div>;
@@ -411,19 +449,31 @@ var ImageBlinker = React.createClass({
     filePair: React.PropTypes.object.isRequired,
     shrinkToFit: React.PropTypes.bool
   },
+  mixins: [SetIntervalMixin],
   getInitialState: function() {
-    return {idx: 0};
+    return {idx: 0, autoBlink: true};
+  },
+  toggleAutoBlink: function(e) {
+    this.setState({
+      autoBlink: $(this.refs.autoblink.getDOMNode()).is(':checked')
+    });
   },
   render: function() {
     var pair = this.props.filePair;
     var side = ['a', 'b'][this.state.idx];
     var path = [pair.a, pair.b][this.state.idx];
     var maxWidth = this.props.shrinkToFit ? window.innerWidth - 30 : null;
-    return <table id="imagediff">
-      <tr className="image-diff-content">
-        <td><AnnotatedImage filePair={pair} side={side} maxWidth={maxWidth} /></td>
-      </tr>
-    </table>;
+    return (
+      <div>
+        <input ref="autoblink" type="checkbox" id="autoblink" checked={this.state.autoBlink} onChange={this.toggleAutoBlink} />
+        <label htmlFor="autoblink"> Auto-blink (hit ‘b’ to blink manually)</label>
+        <table id="imagediff">
+          <tr className="image-diff-content">
+            <td><AnnotatedImage filePair={pair} side={side} maxWidth={maxWidth} /></td>
+          </tr>
+        </table>
+      </div>
+    );
   },
   componentDidMount: function() {
     var toggle = () => {
@@ -435,9 +485,16 @@ var ImageBlinker = React.createClass({
     $(document).on('keydown.blink', (e) => {
       if (!isLegitKeypress(e)) return;
       if (e.keyCode == 66) {  // 'b'
-        toggle();
+        if (this.isMounted()) {
+          this.setState({autoBlink: false});
+          toggle();
+        }
       }
     }).on('click.blink', 'a[value="blink"]', toggle);
+
+    this.setInterval(() => {
+      if (this.state.autoBlink) toggle();
+    }, 500);
   },
   componentDidUnmount: function(e) {
     $(document).off('keydown.blink').off('click.blink');
@@ -520,10 +577,20 @@ var ImageSwipe = React.createClass({
         'background-position-x': -Math.floor(frac * imB.width) + 'px'
       });
     } else {
-      _.extend(styleA, {opacity: (1.0 - frac)});
       _.extend(styleB, {opacity: frac});
     }
 
+    // Add an opaque grid under each image to expose transparency.
+    [styleA, styleB].forEach(function(o) {
+      o['background-image'] += ', url(/static/img/trans_bg.gif)';
+      if (_.has(o, 'background-size')) {
+        o['background-size'] += ', auto auto';
+      }
+      if (_.has(o, 'background-position-x')) {
+        o['background-position-x'] += ', ' + o['background-position-x'];
+      }
+    });
+
     return (
       <div>
         <input type="range" min="0" max={rangeMax} defaultValue={rangeMax/2} ref="slider" onChange={this.onSlide} />
